==> asset-smasher-0.2.7.tgz: package/lib/compilation/transformer.js <==
/**
 *
 * This operation runs an asset through a set of transformations,
 * based on file extensions.
 *
 * E.g
 *
 *   foo.js.coffee.ejs
 *
 * First runs through the EJS preprocessor, then the coffee-script compiler.
 *
 */
var async = require('async');
var path = require('path');
var _ = require('underscore');

// Use setImmediate if available
var setImmediateCompat = global.setImmediate || process.nextTick;

exports.transforms = {
  CoffeeScript:require('./transforms/coffee_transform'),
  Dust:require('./transforms/dust_transform'),
  Ejs:require('./transforms/ejs_transform'),
  Handlebars:require('./transforms/handlebars_transform'),
  Less:require('./transforms/less_transform'),
  Stylus:require('./transforms/stylus_transform'),
  Mf:require('./transforms/mf_transform')
};
exports.postTransforms = {
  /**
   * Ensure that .js files end in a semicolon
   */
  EndJsSemicolon:function (asset, cb) {
    if (path.extname(asset.logicalName) === '.js') {
      var contents = asset.contents;
      if (Buffer.isBuffer(contents)) {
        contents = contents.toString('utf-8');
      }
      if (!/\s*;[\s\n]*$/.test(contents)) {
        contents += ';\n';
      }
      if (!/\n$/.test(contents)) {
        contents += '\n';
      }
      asset.contents = contents;
    }
    cb();
  }
};

var Transformer = exports.Transformer = function (options) {
  options = options || {};
  this.transforms = options.transforms || [];
  this.postTransforms = options.postTransforms || [];
};

Transformer.prototype = {
  asOperation:function () {
    return this.execute.bind(this);
  },
  asDryRunOperation:function () {
    return this.executeDry.bind(this);
  },
  asGlobalDryRunOperation:function () {
    return this.executeGlobalDry.bind(this);
  },
  execute:function (asset, cb) {
    var transforms = this.transforms;
    var postTransforms = this.postTransforms;
    var currentTransform = null;
    // Reset the logical name
    asset.logicalName = path.basename(asset.assetFilePath);
    // Transform the asset
    async.waterfall([
      function (wfCb) {
        // Load the asset
        if (!asset.contents) {
          asset.loadOriginalContents(wfCb);
        }
        else {
          wfCb();
        }
      },
      function (wfCb) {
        // Keep transforming while there are transformers to execute on the file
        async.whilst(function () {
          for (var i = 0; i < transforms.length; ++i) {
            if (transforms[i].shouldTransform(asset.logicalName)) {
              currentTransform = transforms[i];
              return true;
            }
          }
          return false;
        }, function (whilstCb) {
          currentTransform.transform(asset, whilstCb);
        }, wfCb);
      },
      function (wfCb) {
        // Do any post-transform processing that needs to be done
        async.eachSeries(postTransforms, function (postTransform, eachCb) {
          setImmediateCompat(function() {
            postTransform(asset, eachCb);
          });
        }, wfCb);
      }
    ], function (e) {
      cb(e, asset);
    });
  },
  executeGlobalDry:function (assetBundle, cb) {
    var self = this;
    async.eachLimit(assetBundle.getAllAssets(), 50, function(asset, eachCb) {
      self.executeDry(asset, eachCb);
    }, function (e) {
      cb(e, assetBundle);
    });
  },
  executeDry:function (asset, cb) {
    var transforms = this.transforms;
    var currentTransform = null;
    // Reset the logical name
    asset.logicalName = path.basename(asset.assetFilePath);
    // Just transform the name
    async.whilst(function () {
      for (var i = 0; i < transforms.length; ++i) {
        if (transforms[i].shouldTransform(asset.logicalName)) {
          currentTransform = transforms[i];
          return true;
        }
      }
      return false;
    }, function (whilstCb) {
      asset.logicalName = currentTransform.transformedFileName(asset.logicalName);
      whilstCb();
    }, function (e) {
      cb(e, asset);
    });
  },
  getExtensions:function () {
    return _.flatten(_.map(this.transforms, function (transform) {
      return transform.extensions();
    }));
  }
};

==> asset-smasher-0.2.10.tgz: package/lib/compilation/transformer.js <==
/**
 *
 * This operation runs an asset through a set of transformations,
 * based on file extensions.
 *
 * E.g
 *
 *   foo.js.coffee.ejs
 *
 * First runs through the EJS preprocessor, then the coffee-script compiler.
 *
 */
var async = require('async');
var path = require('path');
var _ = require('underscore');

// Use setImmediate if available
var setImmediateCompat = global.setImmediate || process.nextTick;

exports.transforms = {
  CoffeeScript:require('./transforms/coffee_transform'),
  Dust:require('./transforms/dust_transform'),
  Ejs:require('./transforms/ejs_transform'),
  Handlebars:require('./transforms/handlebars_transform'),
  Less:require('./transforms/less_transform'),
  Stylus:require('./transforms/stylus_transform'),
  Mf:require('./transforms/mf_transform')
};
exports.postTransforms = {
  /**
   * Ensure that .js files end in a semicolon
   */
  EndJsSemicolon:function (asset, cb) {
    if (path.extname(asset.logicalName) === '.js') {
      var contents = asset.contents;
      if (Buffer.isBuffer(contents)) {
        contents = contents.toString('utf-8');
      }
      if (!/\s*;[\s\n]*$/.test(contents)) {
        contents += ';\n';
      }
      if (!/\n$/.test(contents)) {
        contents += '\n';
      }
      asset.contents = contents;
    }
    cb();
  }
};

var Transformer = exports.Transformer = function (options) {
  options = options || {};
  this.transforms = options.transforms || [];
  this.postTransforms = options.postTransforms || [];
};

Transformer.prototype = {
  asOperation:function () {
    return this.execute.bind(this);
  },
  asDryRunOperation:function () {
    return this.executeDry.bind(this);
  },
  asGlobalDryRunOperation:function () {
    return this.executeGlobalDry.bind(this);
  },
  execute:function (asset, cb) {
    var transforms = this.transforms;
    var postTransforms = this.postTransforms;
    var currentTransform = null;
    // Reset the logical name
    asset.logicalName = path.basename(asset.assetFilePath);
    // Transform the asset
    async.waterfall([
      function (wfCb) {
        // Load the asset (if it's not a manifest file/dir)
        if (!asset.isManifest() && !asset.contents) {
          asset.loadOriginalContents(wfCb);
        }
        else {
          wfCb();
        }
      },
      function (wfCb) {
        // Keep transforming while there are transformers to execute on the file
        async.whilst(function () {
          for (var i = 0; i < transforms.length; ++i) {
            if (transforms[i].shouldTransform(asset.logicalName)) {
              currentTransform = transforms[i];
              return true;
            }
          }
          return false;
        }, function (whilstCb) {
          currentTransform.transform(asset, whilstCb);
        }, wfCb);
      },
      function (wfCb) {
        // Do any post-transform processing that needs to be done
        async.eachSeries(postTransforms, function (postTransform, eachCb) {
          setImmediateCompat(function() {
            postTransform(asset, eachCb);
          });
        }, wfCb);
      }
    ], function (e) {
      cb(e, asset);
    });
  },
  executeGlobalDry:function (assetBundle, cb) {
    var self = this;
    async.eachLimit(assetBundle.getAllAssets(), 50, function(asset, eachCb) {
      self.executeDry(asset, eachCb);
    }, function (e) {
      cb(e, assetBundle);
    });
  },
  executeDry:function (asset, cb) {
    var transforms = this.transforms;
    var currentTransform = null;
    // Reset the logical name
    asset.logicalName = path.basename(asset.assetFilePath);
    // Just transform the name
    async.whilst(function () {
      for (var i = 0; i < transforms.length; ++i) {
        if (transforms[i].shouldTransform(asset.logicalName)) {
          currentTransform = transforms[i];
          return true;
        }
      }
      return false;
    }, function (whilstCb) {
      asset.logicalName = currentTransform.transformedFileName(asset.logicalName);
      whilstCb();
    }, function (e) {
      cb(e, asset);
    });
  },
  getExtensions:function () {
    return _.flatten(_.map(this.transforms, function (transform) {
      return transform.extensions();
    }));
  }
};
